/*
 * This file is part of libphidget22java
 *
 * Copyright 2015 Phidgets Inc <patrick@phidgets.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see
 * <http://www.gnu.org/licenses/>
 */

#include "phidget_jni.h"
#include "com_phidget22_Phidget.h"
#include "com_phidget22_PhidgetBase.h"

extern const char * CCONV channelInfo(PhidgetHandle, char *, uint32_t);
extern const char * CCONV deviceInfo(PhidgetHandle, char *, uint32_t);

extern jfieldID phidget_handle_fid;
jclass phid_class;
jmethodID phid_cons;

void
com_phidget22_Phidget_OnLoad(JNIEnv *env) {

	if (!(phid_class = (*env)->FindClass(env, "com/phidget22/Phidget")))
		JNI_ABORT_STDERR("Coulnd't FindClass com/phidget22/Phidget");
	if (!(phid_class = (jclass)(*env)->NewGlobalRef(env, phid_class)))
		JNI_ABORT_STDERR("Couldn't create global ref phid_class");
	if (!(phid_cons = (*env)->GetMethodID(env, phid_class, "<init>", "(J)V")))
		JNI_ABORT_STDERR("Couldn't get Method ID <init> from phid_class");
}

JNIEXPORT jstring JNICALL
Java_com_phidget22_Phidget_getDescription(JNIEnv * env, jobject obj) {
	PhidgetHandle h = (PhidgetHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	int channel;
	char buf[128];

	if (!h) {
		PH_THROW(EPHIDGET_UNEXPECTED);
		return NULL;
	}

	if (Phidget_getIsChannel(h, &channel)) {
		PH_THROW(EPHIDGET_UNEXPECTED);
		return NULL;
	}

	if (channel)
		return (*env)->NewStringUTF(env, channelInfo(h, buf, sizeof(buf)));
	return (*env)->NewStringUTF(env, deviceInfo(h, buf, sizeof(buf)));
}

JNIEXPORT void JNICALL
Java_com_phidget22_Phidget_deleteHandle(JNIEnv * env, jobject obj) {
	PhidgetHandle h;

	h = (PhidgetHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	Phidget_delete(&h);
}

JNIEXPORT jobject JNICALL Java_com_phidget22_PhidgetBase_getParent(JNIEnv * env, jobject obj) {
	PhidgetHandle parent;
	PhidgetHandle h;
	int error;

	h = (PhidgetHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	if ((error = Phidget_getParent(h, &parent))) {
		PH_THROW_LAST_ERR(error);
		return NULL;
	}

	if (parent == NULL)
		return (NULL);

	return ((*env)->NewObject(env, phid_class, phid_cons, parent));
}

JNIEXPORT jobject JNICALL Java_com_phidget22_PhidgetBase_getHub(JNIEnv * env, jobject obj) {
	PhidgetHandle hub;
	PhidgetHandle h;
	int error;

	h = (PhidgetHandle)(uintptr_t)(*env)->GetLongField(env, obj, phidget_handle_fid);
	if ((error = Phidget_getHub(h, &hub))) {
		PH_THROW_LAST_ERR(error);
		return NULL;
	}

	if (hub == NULL)
		return (NULL);
	return ((*env)->NewObject(env, phid_class, phid_cons, hub));
}